require 'uu/os/gvc/container'
require 'uu/os/gvc/message'
require 'uu/os/form/input'
require 'uu/os/gvc/component'
require 'uu/os/vuc/visual_use_case_error'
require 'uu/os/vuc/error_codes'

module UU
  module OS
    module VUC

      # @private
      class ComponentContainer < UU::OS::GVC::Container

        def initialize(ctx)
          @ctx = ctx
          @components = {}
          @messages = []
          @focus_on = nil
          # TODO do not want init all components!!
          init_components
        end

        def get_component_by_id(id)
          component_by_code(id)
        end

        def get_components_by_code(code)
          result = []
          result << component_by_code(code)
          return result
        end

        def component_by_code(code)
          @components[code]
        end

        def components
          @components.values
        end

        def get_components
          @components
        end

        def add_message(message)
          msg = UU::OS::GVC::Message.new(message)
          @messages << msg
        end

        def messages
          @messages
        end

        def focus(id)
          @components.each do |k, comp|
            if comp.id == id
              focus_by_code(k)
            end
          end
        end

        def focus_by_code(component_code)
          component_code = component_code.to_sym if component_code.kind_of? String
          return false if !component_by_code(component_code)
          @focus_on = component_code
          return true
        end

        def focused_component
          @components[@focus_on]
        end

        def get_focus
          @focus_on
        end

        private

        # @private
        def init_components
          return if @ctx.nil? || @ctx.parameters.nil?
          @ctx.parameters.each do |key, data|
            if (data.nil?) || (!data.kind_of? Hash) || (data[:componentType].nil?)
              next
            end
            handle_duplicit_component_code(@ctx.parameters) if key.to_s.match(/^uuDuplicit/)
            @components[key] = create_component(data)
          end
        end

        # @private
        def create_component(data)
          if data
            comp_cls = get_component_class(data[:componentType])
            if comp_cls
              # Create component according to mapping
              return comp_cls.new(self, data)
            elsif !data[:value].nil? || !data[:data].nil?
              # If no mapping exists but component contains value or data, it is probably Input component
              return UU::OS::Form::Input.new(self, data)
            end
          end
          # Create generic component by default
          UU::OS::GVC::Component.new(self, data)
        end

        # @private
        def get_component_class(componentType)
          return if componentType.nil?
          componentType = componentType.to_s.downcase
          component_map = UU::OS::GVC::Container::class_variable_get(:@@COMPONENT_MAPPING)
          return component_map[componentType]
        end

        # @private
        def handle_duplicit_component_code(params)
          return if params.nil?

          duplicities = []

          original_code = ''
          params.each do |key, data|
            match = key.to_s.match(/^uuDuplicit-[0-9]+-(.*)/)
            if match
              original_code = match[1]
               break
            end
          end

          params.each do |key, data|
            if key.to_s.match(/^uuDuplicit/) || (key.to_s == original_code)
              duplicities << "Component with code : #{key}, component data : #{data.to_s}"
            end

          end
          raise UU::OS::VUC::VisualUseCaseError.new(UU::OS::IPC::ErrorKind::SERVER, UU::OS::VUC::ErrorCodes::COMPONENT_CODE_DUPLICITY,
              "Duplicite code deteced. There are compoennts with same code which is prohibited. Duplicities: #{duplicities.to_s}")
        end

      end
    end
  end
end
